@@ -1,9 +1,11 @@
 use crate::point::{Point, Vec2DWorld, Vec2DScreen};
-use crate::draw_commands::DrawCommand;
+use crate::draw_commands::{DrawCommand, circle_helper};
 use crate::color::Color;
 use crate::transform::Transform;
+use crate::path_command::PathCommand;
+use crate::geom::middle_point;
 
-use super::{ShapeBuilder, ShapeFinished, path::{PathCommand, Path}};
+use super::{ShapeBuilder, ShapeFinished, path::Path};
 
 #[derive(Debug)]
 pub struct AxisAlignedRectangleBuilder {
@@ -26,7 +28,7 @@ impl AxisAlignedRectangleBuilder {
     }
 }
 
-fn draw_commands(corners: [Vec2DWorld; 4]) -> Vec<PathCommand> {
+fn draw_commands(corners: [Vec2DWorld; 4]) -> Vec<PathCommand<Vec2DWorld>> {
     let mut commands = Vec::with_capacity(5);
     let mut points_iter = corners.iter().cycle().take(5);
 
@@ -55,18 +57,24 @@ impl ShapeBuilder for AxisAlignedRectangleBuilder {
         self.handle_mouse_moved(pos, t, snap);
 
         ShapeFinished::Yes(Box::new(Path::from_parts(
-            draw_commands(self.all_four_corners()), self.color, self.stroke,
+            draw_commands(self.all_four_corners()), Some(self.color), None, self.stroke,
         )))
     }
 
-    fn draw_commands(&self) -> Vec<DrawCommand> {
+    fn draw_commands(&self, t: Transform, snap: f64) -> Vec<DrawCommand> {
         let corners = self.all_four_corners();
+        let center = middle_point(corners[0], corners[2]);
 
-        vec![DrawCommand::Path {
-            commands: draw_commands(corners),
-            thickness: self.stroke,
-            color: self.color,
-        }]
+        vec![
+            DrawCommand::Path {
+                commands: draw_commands(corners),
+                thickness: self.stroke,
+                color: Some(self.color),
+                fill: None,
+            },
+
+            circle_helper(t.to_screen_coordinates(center), snap),
+        ]
     }
 }
 
@@ -87,8 +95,9 @@ mod tests {
         rect.handle_mouse_moved(b, t, SNAP);
 
         // while being drawn
-        assert_eq!(rect.draw_commands()[0], DrawCommand::Path {
-            color: Default::default(),
+        assert_eq!(rect.draw_commands(t, SNAP)[0], DrawCommand::Path {
+            color: Some(Default::default()),
+            fill: None,
             thickness: 1.0,
             commands: vec![
                 PathCommand::MoveTo(Vec2DWorld::new(4.0, 5.0)),
@@ -103,7 +112,8 @@ mod tests {
         match rect.handle_button_released(b, t, SNAP) {
             ShapeFinished::Yes(s) => {
                 assert_eq!(s.draw_commands(), DrawCommand::Path {
-                    color: Default::default(),
+                    color: Some(Default::default()),
+                    fill: None,
                     thickness: 1.0,
                     commands: vec![
                         PathCommand::MoveTo(Vec2DWorld::new(4.0, 5.0)),
